/*

  (C) 1997 Marco Falcioni UCLA

  build and print coordiantion sequence (CS)

*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "error.h"

#include "grid.h"
#include "box.h"
#include "cell.h"
#include "positions.h"
#include "atoms.h"
#include "link.h"
#include "chain.h"
#include "oogl.h"
#include "misc.h"
#include "params.h"

#define DEBUG

void fill_shell(struct Chain * C1, 
		const struct Chain * C2, const struct Grid * G,
		const struct Cell * C,
		const struct Params * P);


void build_CS (const struct Params * P, struct Grid * Gr, 
	       const struct Positions * T, const struct Cell * C, 
	       unsigned long depth) {
  struct Chain ** CS;
  struct Link * L;
  unsigned long i;

  if(Gr == 0 || C == 0 || T == 0 || depth == 0) {
    push_generic_error("build_CS","(nil) or zero args passed");
    exit_clean();
  }

  if(T->merged != 1 || Gr->filled != 1) {
    char buf[1024];
    sprintf(buf, "Grid or Positions not ready: Gr->filled %d T->merged %d",
	    Gr->filled,T->merged );
    push_generic_error("build_CS",buf); 
    exit_clean();
  }

  CS = (struct Chain **) Malloc(depth * sizeof(struct Chain *));

  for(i = 0; i < depth; i++) 
    CS[i] = new_Chain();
  
  /* 
     we only need to build the coordination sequence for the unique atoms
  */

  for(L = T->unique->head; L != 0; L = L->next) {
    struct Link * N;

    N = new_Link(L->atom); /* add first Link to coordination sequence */ 
    add_Link(N, CS[0]);
    L->atom->tag = 1; /* GREAT UGLINESS */

#ifdef DEBUG    
    printf("T %ld",CS[0]->length);
#endif

    for(i = 0; i < depth-1; i++) {
      fill_shell(CS[i+1], CS[i], Gr, C, P);
#ifdef DEBUG
      printf(" %ld",CS[i+1]->length); 
#endif
    }

#ifdef DEBUG
    printf("\n");
#endif

    untag_Grid(Gr);

    for(i = 0; i < depth; i++) {
      clean_Chain(CS[i]);
    }
  }

  for(i = 0; i < depth; i++) {
    delete_Chain(CS[i]);
  }

  Free(CS);
  return;
}

void fill_shell(struct Chain * C1, 
		const struct Chain * C2, const struct Grid * G,
		const struct Cell * C, const struct Params * P)
{
  struct Link * curlink;

  if(C2->length == 0) return; /* nothing to do */
  
  for(curlink = C2->head; curlink != 0; curlink = curlink->next) {
    struct Link * testme;
    struct Box * curbox;
    long i;

    curbox = index_atom_Grid(G, curlink->atom, C, 0);

    for(testme = curbox->list->head; testme != 0; 
	testme = testme->next) {
      double dist2;

      if(testme->atom->tag != 1 && testme != curlink) { 

	dist2 = dist2_xyz(C,curlink->atom, testme->atom);
	
	if(dist2 < P->bond2) {
	  struct Link * newone;
	  newone = new_Link(testme->atom);
	  testme->atom->tag = 1;
	  add_Link(newone, C1);
	}
      }
      
    }

    /*  test all neighboring boxes */


    for(i = 0; i < curbox->n; i++) {
      struct Box * otherbox = curbox->near[i];

      for(testme = otherbox->list->head; testme != 0; 
	  testme = testme->next) {
	double dist2;
	if(testme->atom->tag != 1) {

	  dist2 = dist2_xyz(C,curlink->atom, testme->atom);
	  if(dist2 < P->bond2) {
	    struct Link * newone;
	    newone = new_Link(testme->atom);
	    testme->atom->tag = 1;
	    add_Link(newone, C1);
	  }
	}
      }
    }
  }


  return;
}
