/*

  Marco Falcioni (C) 1997

  cell data, properties and methods


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __cell_h
#define __cell_h

/* includes */
#include <stdio.h>


/* data */
struct Atom; 

struct Cell {
  double a,b,c;
  double alpha, beta, gamma;
  double cosalpha, cosbeta, cosgamma; /* speed */
  double ** xyz_to_abc;
  double ** abc_to_xyz;
  double ** rec_vec; /* [0] is C_k, [1] is C_l, [2] is C_m */
  double ** transl; /* all possible translations to neighboring cells */
  double lnl[3]; /* lower near right corner */
  double ufr[3]; /* upper far left corner: 
		    this means that no matter what, lnl has all the mins
		    and ufr has all the maxes */
  int csym; /* is the cell centrosymetric? */
};

/* function prototypes */

struct Cell * new_Cell(const double a, const double b, const double c, 
		       const double alpha, const double beta, 
		       const double gamma);

void delete_Cell(struct Cell * C);


struct Cell * read_Cell(FILE * stream);
struct Cell * read_Cell_XTL(FILE * stream);

void write_Cell(const struct Cell * C, FILE * stream);
void write_Cell_XTL(const struct Cell * C, FILE * stream);

/* returns the cell volume */
double volume_Cell (const struct Cell * C);

/* returns the distance between two points in real units */
double dist2_xyz(const struct Cell * C, 
		 struct Atom * A, struct Atom * B);

/* returns the minimum distance between two points in real units */
double min_dist2_xyz(const struct Cell * C, 
		const struct Atom * A, const struct Atom * B);

/* retuns the interplanar spacing of a given plane [ijk] */
double dist2_plane (const struct Cell * C, 
		   const long i, const long j, const long k);

#endif
