/*

  (C) 1998 Marco Falcioni UCLA

  generic implementation file

*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>

extern int errno;

#include "error.h"

#include "misc.h"

#include "conf.h"
#include "positions.h"
#include "atoms.h"
#include "link.h"
#include "chain.h"

double get_next_number(char ** str_ptr);

struct Conf * new_Conf(unsigned long unique)
{
  unsigned long i;
  struct Conf * me;

  me = (struct Conf *) Malloc (sizeof(struct Conf));

  me->n_unique = unique;

  /* do some initialization */

  for(i = 0; i < ENERGIES; i++) {
    me->ene[i] = (double *) malloc(unique * sizeof(double));
  }

  for(i = 0; i < 3; i++) {
    me->pos[i] = (double *) malloc(unique * sizeof(double));
  }

  me->next = 0;

  return me;
}

void delete_Conf(struct Conf * me)
{
  unsigned long i;
  if(me == 0) return;

  for(i = 0; i < ENERGIES; i++) {
    Free(me->ene[i]);
  }

  for(i = 0; i < 3; i++) {
    Free(me->pos[i]);
  }

  Free(me);

  return;
}

int read_Conf(struct Conf * conf, FILE * stream)
{
  char line[6096];
  char * p;
  unsigned i = 0;
  
#ifndef NOCHECKS
  if(conf == 0 || stream == 0) {
    push_generic_error("read_Conf", "(nil) args passed");
    exit_clean();
  }
#endif

  p = get_line(line, sizeof(line), stream);

  if(feof(stream) || p == 0) return -1;

#ifdef DEBUG
  printf("line is: %s\n", p);
#endif
  /* skip the first line number */
  (void) get_next_number(&p);
  
  /* read three energies */
  conf->tot_energy = get_next_number(&p);
  conf->RP_energy = get_next_number(&p);
  conf->Pos_energy = get_next_number(&p);
  
  /* now read individual energies and positions */
  
  for(i = 0; i < conf->n_unique; i++) {
    unsigned j;
    for(j = 0; j < ENERGIES; j++) {
      conf->ene[j][i] = get_next_number(&p);
    }
#ifdef DEBUG
      printf("read pos atom %d\n", i);
#endif    
    conf->pos[0][i] = get_next_number(&p);
    conf->pos[1][i] = get_next_number(&p);
    conf->pos[2][i] = get_next_number(&p);
  }
  return 0;
}




double get_next_number(char ** str_ptr)
{
  unsigned i = 0;
  char word[50];
  double number = DBL_MAX;
#ifdef DEBUG
  printf("someone called me!\n");
#endif
  while(!isspace((int) **str_ptr) && **str_ptr != '\n') {
    word[i] = **str_ptr;
    word[i+1] = '\0';
    i++;
    (*str_ptr)++;
  }
  i = sscanf(word,"%lf", &number);

#ifdef DEBUG
    printf("and word %s number %f\n", word, number);
    fflush(stdout);
#endif

  if (i != 1) { 
    char buf[1024];
    printf(buf,"error reading conf at line: %s", *str_ptr);
    push_generic_error("get_next_number", buf);
    exit_clean();
  }

  while(isspace((int) **str_ptr)) (*str_ptr)++;

  return number;
  
}


void copy_Conf(struct Conf * a, const struct Conf * b)
{
  int i;
#ifndef NOCHECKS
  if(a == 0 || b == 0) {
    push_generic_error("copy_Conf","(nil) args passed");
    exit_clean();
  }
#endif

  a->n_unique = b->n_unique;

  a->Pos_energy = b->Pos_energy;
  a->RP_energy = b->RP_energy;
  a->tot_energy = b->tot_energy;

  for(i = 0; i < (int) a->n_unique; i++) {
    int j;
    for(j = 0; j < ENERGIES; j++) a->ene[j][i] = b->ene[j][i];
    for(j = 0; j < 3; j++) a->pos[j][i] = b->pos[j][i];
  }

  return;
}

void pcopy_Conf(struct Conf * me, struct Positions * T)
{
  struct Link * runner;
  unsigned atom = 0;

#ifndef NOCHECKS
  if(me == 0 || T == 0) {
    push_generic_error("pcopy_Conf","(nil) args passed");
    exit_clean();
  }
#endif

  for(runner = T->unique->head; runner != 0; runner = runner->next) {
    double pos[3];

    pos[0] = me->pos[0][atom];
    pos[1] = me->pos[1][atom];
    pos[2] = me->pos[2][atom];

    move_Atom(runner->atom, pos);

    atom ++;
  }      

  T->test = 0; 
  T->mod = 0;
  T->grid = 0;
  T->merged = 0;  

  return;
}


void print_Conf(struct Conf * me, FILE * stream)
{
  unsigned long i;
#ifndef NOCHECKS
  if(me == 0 || stream == 0) {
    push_generic_error("print_Conf","(nil) args passed");
    exit_clean();
  }
#endif

  /* do some fprintfing */
  fprintf(stream,"Conf at %p\n", me);
  fprintf(stream,"Energy %f + %f = %f\n", me->RP_energy, 
	  me->Pos_energy, me->tot_energy);
  for(i = 0; i < me->n_unique; i++) {
    int j;
    fprintf(stream, "Atom %ld: enes ", i);
    
    for(j = 0; j < ENERGIES; j++)
      fprintf(stream,"%f ", me->ene[j][i]);

    fprintf(stream,"%f %f %f\n",
	    me->pos[0][i], me->pos[1][i], me->pos[2][i]);
  }

  if(ferror(stream)){
    char buf[1024];
    sprintf(buf, "error while writing to stream: %s", strerror(errno));
    push_generic_error("print_Conf",buf);
    exit_clean();
  }

}   

void sprint_Conf(struct Conf * me, char * buffer)
{
  unsigned long i;
  char buf[1024];
#ifndef NOCHECKS
  if(me == 0 || buffer == 0) {
    push_generic_error("sprint_Conf","(nil) args passed");
    exit_clean();
  }
#endif

  /* this is broken */
  exit_clean();

  sprintf(buf, "%f %f %f", me->tot_energy, me->RP_energy, me->Pos_energy);
  strcat(buffer, buf);
  for(i = 0; i < me->n_unique; i++) {
    int j;
    for(j = 0; j < ENERGIES; j++) {
      sprintf(buf, " %f", me->ene[j][i]);
      strcat(buffer, buf);
    }
    sprintf(buf, " %f %f %f", me->pos[0][i], me->pos[1][i], me->pos[2][i]);
    strcat(buffer, buf);
  }

  return;
}

void fprint_Conf(struct Conf * me, FILE * stream)
{
  unsigned long i;
#ifndef NOCHECKS  
  if(me == 0 || stream == 0) {
    push_generic_error("fprint_Conf","(nil) args passed");
    exit_clean();
  }
#endif
  fprintf(stream, "%f %f %f", me->tot_energy, me->RP_energy, me->Pos_energy);
  
  for(i = 0; i < me->n_unique; i++) {
    int j;
    
    for(j = 0; j < ENERGIES; j++)
      fprintf(stream," %f", me->ene[j][i]);

    fprintf(stream," %f %f %f",
	    me->pos[0][i], me->pos[1][i], me->pos[2][i]);
  }

  fprintf(stream, "\n");
  fflush(stream);
  return;
}
