/* Created: [GJ] Mon Feb 10 06:36:48 EST 1997
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "universal_io.h"
#include "error.h"
#include "memory.h"

static char * progname;
static char err_message[1024] = "";
static int err_code = Error_None_;
static FILE * err_str = (FILE *)0;

/* local data for error explanations */
struct explain_item { int c; char *s; };
static struct explain_item error_explains[] = {
  { Error_None_,       "No Error" },
  { Error_General_,    "Unspecified Error"},
  { Error_Domain_,     "Domain error in function evaluation" },
  { Error_ConvFail_,   "Converegence failure in iterative evaluation" },
  { Error_Alloc_,      "Allocation failure in memory allocation attempt"},
  { -1, "_no_message_"}
};

static char empty_message[2] = "";


int connect_error_stream(FILE *fp, char * name) {
  if(fp == 0 || name == 0) {
    return 1;
  }
  else {
    progname = name;
    err_str = fp;
    return 0;
  }
}


int disconnect_error_stream(void) {
  err_str = (FILE *) 0;
  return 0;
}


int error_code(void) { return err_code; }


const char * error_explain(int code) {
  int i = 0;
  int find_code = 0;
  while((code = error_explains[i].c) != -1) {
    if(code == find_code) {
      return error_explains[i].s;
    }
  }
  return empty_message;
}


void push_error(const char * mess, int code)
{
  if(mess != 0) {
    int len_mess = strlen(mess);
    int len = (len_mess > 1024 ? 1024 : len_mess);
    strncpy(err_message, mess, len+1);
    err_code = code;

    if(err_str != (FILE *)0) {
      fprintf(err_str,"%s: ERROR %5d  %s\n",
	      _LIBNAME_,
	      err_code,
	      err_message
	      );
      fflush(err_str);
    }
  }
}


void push_generic_error(const char * prefix, const char * mess)
{
  int len_mess = strlen(mess);
  int len = (len_mess > 1024 ? 1024 : len_mess);
  strncpy(err_message, mess, len+1);
  err_code = Error_General_;

  if(err_str != (FILE *)0) {
    fprintf(err_str,"%s.%s: ERROR: %s\n",
	    progname,
	    prefix,
	    err_message
	    );
    fflush(err_str);
  }
}

void push_generic_warning(const char * prefix, const char * mess)
{
  int len_mess = strlen(mess);
  int len = (len_mess > 1024 ? 1024 : len_mess);
  strncpy(err_message, mess, len+1);
  err_code = Error_General_;

  if(err_str != (FILE *)0) {
    fprintf(err_str,"%s.%s: WARNING: %s\n",
	    progname,
	    prefix,
	    err_message
	    );
    fflush(err_str);
  }
}

void flush_error_stream(void) { if(err_str != (FILE *)0) fflush(err_str); }


void exit_clean(void) {
  flush_error_stream();
  exit(20);
}
