/*

  Marco Falcioni (C) 1997

  general header for all the functions that are not methods


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __functions_h
#define __functions_h

#include <stdio.h> 

struct Cell;
struct Grid;
struct Positions;
struct Sym_group;
struct SP_entry;
struct Rad_pattern;
struct Atom;
struct Params;
struct Conf;
struct Trial;

/* function prototypes */

extern double find_angle(struct Atom * a, 
			 struct Atom * b, 
			 struct Atom * c,
			 double da,
			 double db);



extern double energy(const struct Params * P, 
		     const struct Cell * C, 
		     const struct Positions * T, 
		     const struct Grid * Gr,
		     struct Conf * co);

extern void abc_to_xyz_Positions(struct Positions * T_xyz, 
				 const struct Positions * T_abc, 
				 const struct Cell * C);

extern void build_CS (const struct Params * P, 
		      struct Grid * Gr, 
		      const struct Positions * T, 
		      const struct Cell * C, 
		      unsigned int depth);

extern void symmetrize_Positions(struct Positions * T, 
				 const struct Sym_group * G);

extern void dump_27 (const struct Positions * T_abc, 
		     const struct Cell * C, FILE * stream);

extern void move_atoms (const struct Positions * T_old, 
			struct Positions * T_new, 
			const double radius);

/* 
   read a datafile and return newly allocated positions
   cell and group
   a sample datafile is
   
   2 24  36
   4  4
   
   18.126 18.126  7.567
   90.000 90.000 120.000
   CENTROSYMMETRIC
   1.   0.   0.   0.   1.   0.   0.   0.   1. 0.0   0.0   0.0
   ...

   # comments and whitespace are ignored.
*/
extern void read_data(struct Params ** P, struct Positions ** T, 
		      struct Cell ** C, struct Sym_group ** G,
		      struct Rad_pattern ** RP,
		      FILE * stream);

void merge_positions(struct Positions * T,
		     const struct Params * P,
		     const struct Cell * C);

extern struct SP_entry ** read_SP_file(struct Positions * T, 
				       char * name);

extern void get_correction(struct Rad_pattern * RP,
			   const struct Positions * T,
			   const struct Cell * C, 
			   struct SP_entry ** SP);

void get_intensity(struct Rad_pattern * RP,
		   const struct Positions * T,
		   const struct Cell * C,
                   const struct Sym_group * G);

void find_neighbors(struct Grid * Gr, 
		    struct Positions * T, 
		    struct Cell * C);

void move_atom (struct Positions * T_new, 
		const unsigned int i_atom,
		const struct Trial * Delta,
		const struct Cell * C);

double monte_carlo (const struct Params * P, const struct Cell * C,
		  const struct Sym_group * G, struct Rad_pattern * RP,
		  struct Positions * T1, struct Positions * T2,
		  struct Grid * Gr, struct Conf * Min);

double minimize (const struct Params * P, const struct Cell * C,
		  const struct Sym_group * G, struct Rad_pattern * RP,
		  struct Positions * T1, struct Positions * T2,
		  struct Grid * Gr, struct Conf * Min);

double biased_monte_carlo (const struct Params * P, const struct Cell * C,
		  const struct Sym_group * G, struct Rad_pattern * RP,
		  struct Positions * T1, struct Positions * T2,
		  struct Grid * Gr, struct Conf * Min);

void anneal (struct Params * P, const struct Cell * C,
	     const struct Sym_group * G, struct Rad_pattern * RP,
	     struct Positions * T1);

void biased_anneal (struct Params * P, const struct Cell * C,
	     const struct Sym_group * G, struct Rad_pattern * RP,
	     struct Positions * T1);

void parallel_tempering (struct Params * P, const struct Cell * C,
	     const struct Sym_group * G, struct Rad_pattern * RP,
	     struct Positions * T1);

void find_multiplicity(struct Rad_pattern * RP, const struct Cell * C,
		       double Min_Angle, double Max_Angle, double min_intensity);

void find_composites(struct Rad_pattern * RP, double separation);

void run_ID (FILE * stream);

void write_XTL(struct Positions * T, struct Cell * C, struct Sym_group * G, 
	       FILE * stream);

void write_P1(struct Positions * T, struct Cell * C, struct Sym_group * G, 
	       FILE * stream);

void read_XTL( struct Positions * T, 
	       struct Cell ** C, struct Sym_group ** G, 
	       FILE * stream);
#endif
