/* Created: [GJ] Fri Apr 26 20:53:29 EDT 1996
 *
 * [GJ] Wed Jan 15 21:32:59 EST 1997
 * Inserted use of error stream facility.
 *
 * [GJ] Tue Feb  4 04:06:07 EST 1997
 * Inserted new memory allocation functions.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "error.h"
#include "constants.h"
#include "memory.h"
#include "sorting.h"
#include "interpolation.h"


void interp_poly(const double * xa, const double * ya, int n,
		 double x, double * y, double * dy)
{
  int i,m,ns=1;
  double den,dif,dift,ho,hp,w;
  double *c,*d;

  /* AAARRRGGHHHH!!!!!!!! ... */
  /*
  --xa;
  --ya;
  */

  dif=fabs(x-xa[1]);
  c = new_vector_d(n+1);
  d = new_vector_d(n+1);

  for (i=1;i<=n;i++) {
    if ( (dift=fabs(x-xa[i])) < dif) {
      ns=i;
      dif=dift;
    }
    c[i]=ya[i];
    d[i]=ya[i];
  }
  *y=ya[ns--];
  for (m=1;m<n;m++) {
    for (i=1;i<=n-m;i++) {
      ho=xa[i]-x;
      hp=xa[i+m]-x;
      w=c[i+1]-d[i];
      if ( (den=ho-hp) == 0.0) {
        push_error("interp_poly: convergence failure", Error_ConvFail_);
	free_vector(d);
	free_vector(c);
        return;
      }
      den=w/den;
      d[i]=hp*den;
      c[i]=ho*den;
    }
    *y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
  }
  free_vector(d);
  free_vector(c);
}


/* Note:original Num.Rec. version is sick. The problem is
   the "rare" 0/0 condition. Firstly, the right value of TINY
   is profoundly dependent on the precision of the calculation.
   The original value of 1.e-25 was probably ok for their
   single precision version, but I consistently generated a
   dd==0.0 error with that value, the only difference being my
   explicit double precision. With the choice TINY=1.e-15,
   I eliminated the troublesome dd==0.0 condition with no
   loss of precision or difficulty for the algorithm.
   However, it is clear that this 'TINY' thing is a totally
   stupid way to handle the 0/0 condition. Amongst other problems,
   it means that the y[] data must be much bigger than TINY, or
   their values are getting twiddled. Therefore, as written, this
   algorithm is not invariant under y[] scaling. What a piece of
   shit.
*/
/* #define TINY 1.e-25 */
#define TINY 1.e-15
void interp_rat(const double * xa, const double * ya, int n,
		double x, double * y, double * dy)
{
  int i, m;
  int ns = 1;

  double * c = new_vector_d(n+1);
  double * d = new_vector_d(n+1);

  double h;
  double hh;
  double absx = fabs(x);

  /* AAARRRGGHHHH!!!!!!!! ... */
  --xa;
  --ya;

  hh = fabs(x - xa[1]);

  for(i=1; i<=n; i++){

    h = fabs(x-xa[i]);

    if(h < absx * dbl_precision){
      *y = ya[i];
      *dy = 0.;
      free_vector(d);
      free_vector(c);
      return;
    }
    else if(h < hh){
      ns = i;
      hh = h;
    }
    c[i] = ya[i];
    d[i] = ya[i] + TINY; /* hmmm.... */
  }

  *y = ya[ns--];

  for(m=1; m<n; m++){
    for(i=1; i<=n-m; i++){
      
      double t, dd;
      double w = c[i+1] - d[i];

      h = xa[i+m] - x;
      t = (xa[i] - x) * d[i] / h;
      dd = t - c[i+1];

      if(dd==0.){
	char buff[1024];
	sprintf(buff, "interp_rat: apparent pole at x= %g", x);
	push_error(buff, Error_ConvFail_);
	free_vector(d);
	free_vector(c);
	return;
      }
      dd = w / dd;
      d[i] = c[i+1] * dd;
      c[i] = t * dd;
    }

    *y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
  }

  free_vector(d);
  free_vector(c);
}


void spline(const double x[], const double y[], int n, double y2[])
{
  int i, k;
  double qn, un;
  double * u = new_vector_d(n);

  /* Natural spline condition. */
  y2[0] = u[0] = 0.0;

  for (i=1; i<n-1; i++) {
    double sig = (x[i]-x[i-1])/(x[i+1]-x[i-1]);
    double p = sig * y2[i-1] + 2.0;
    y2[i] = (sig-1.0) / p;
    u[i] = (y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i] = (6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1]) / p;
  }

  /* Natural spline condition. */
  qn = un = 0.0;

  y2[n-1] = (un - qn*u[n-2]) / (qn*y2[n-2] + 1.0);
  for (k=n-2; k>=0; k--) {
    y2[k] = y2[k]*y2[k+1] + u[k];
  }

  free_vector(u);
}

double * new_spline_y2(const double x[], const double y[], int n)
{
  double * y2 = new_vector_d(n);
  spline(x, y, n, y2);
  return y2;
}

void spline_nn(const double x[], const double y[], int n, double y2[])
{
  int i, k;
  double qn, un;
  double * u = new_vector_d(n);

  /* Endpoint condition. Insures that the derivative at the
   * endpoint is equal to (y[1]-y[0])/(x[1]-x[0]).
   */
  y2[0] = -0.5;
  u[0] = 0.;

  for (i=1; i<n-1; i++) {
    double sig = (x[i]-x[i-1])/(x[i+1]-x[i-1]);
    double p = sig * y2[i-1] + 2.0;
    y2[i] = (sig-1.0) / p;
    u[i] = (y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i] = (6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1]) / p;
  }

  /* Endpoint condition. Similar to above. */
  qn = 0.5;
  un = 0.;

  y2[n-1] = (un - qn*u[n-2]) / (qn*y2[n-2] + 1.0);
  for (k=n-2; k>=0; k--)
    y2[k] = y2[k]*y2[k+1] + u[k];

  free_vector(u);
}


double *new_spline_nn_y2(const double x[], const double y[], int n)
{
  double * y2 = new_vector_d(n);
  spline_nn(x, y, n, y2);
  return y2;
}


double interp_spline(const double xa[], const double ya[],
		     const double y2a[], int n,
		     double x, int *rklo, int *rkhi)
{
  int k;
  double h,b,a;

  int klo = 0;
  int khi = n-1;

  if(x < xa[0] || x > xa[n-1]){
    push_error("interp_spline: x outside x[] range", Error_Domain_);
    return 0.;
  }

  while( khi-klo > 1 ) {
    k = (khi+klo) >> 1;
    if (xa[k] > x) khi = k;
    else klo = k;
  }

  h = xa[khi] - xa[klo];
  if (h < 1.e-30) {
    push_error("interp_spline: bad x[] data input", Error_General_);
    return 0.;
  }

  a = (xa[khi]-x) / h;
  b = (x-xa[klo]) / h;

  *rklo = klo;
  *rkhi = khi;

  return a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}


double interp_spline_quick(const double xa[], const double ya[],
			   const double y2a[],
			   int klo, int khi,
			   double x)
{
  double h = xa[khi] - xa[klo];
  double a = (xa[khi]-x) / h;
  double b = (x-xa[klo]) / h;

  return a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}


#define DefaultCacheSize_ 20
double interp_spline_cached(const double xa[], const double ya[],
			    const double y2a[], int n,
			    double x,
			    int *klocache, int *khicache)
{
  double h,b,a;
  int klo;
  int khi;

  /* If the cache does not exist, then make it point
   * to the first place in the full array, with a
   * given or default size.
   */
  if(klocache >= khicache){
    *klocache = 0;
    *khicache = Min(DefaultCacheSize_, n);
  }

  if(x > xa[*klocache] && x < xa[*khicache]){

    /* Cache hit. Bisection search on the cache. */
    int k;
    klo = *klocache;
    khi = *khicache;

    while( khi-klo > 1 ) {
      k = (khi+klo) >> 1;
      if (xa[k] > x) khi = k;
      else klo = k;
    }
  }
  else {
    
    /* Cache miss. Bisection search on full xa[]. */
    int k;
    klo = 0;
    khi = n - 1;
    
    if(x < xa[0] || x > xa[n-1]){
      push_error("interp_spline_cached: x outside x[] range", Error_Domain_);
      return 0.;
    }
    while( khi-klo > 1 ) {
      k = (khi+klo) >> 1;
      if (xa[k] > x) khi = k;
      else klo = k;
    }
 
    /* Rebuild the cache, centered around the new point. */
    *klocache = Max(klo - DefaultCacheSize_/2, 0);
    *khicache = Min(khi + DefaultCacheSize_/2, n-1);
  }

  /* Evaluate the interpolation. */
  h = xa[khi] - xa[klo];
  if (h < 1.e-50) {
    push_error("interp_spline_cached: bad x[] input", Error_General_);
    return 0.;
  }
  a = (xa[khi]-x) / h;
  b = (x-xa[klo]) / h;

  return a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}
#undef DefaultCacheSize_


void spline_deriv(const double * x, const double * y, int n, double * y_prime)
{
  if(n > 4){
    int i;
    double * f = new_vector_d(n);
    double * g = new_vector_d(n);

    /* Boundary points. Quartic fit. */
    f[0] = (-10.*y[0] + 15.*y[1] - 6.*y[2] + y[3]) / 6.;
    f[n - 1] = (10.*y[n - 1] - 15.*y[n - 2] + 6.*y[n - 3] - y[n - 4]) / 6.;

    /* Interior points. */
    g[0] = 0.;
    for(i=1; i < n - 1; i++){
      g[i] = 1. / (4. - g[i-1]);
      f[i] = g[i] * (3.*(y[i+1] - y[i-1]) - f[i-1]);
    }
    y_prime[n - 1] = f[n - 1];
    for(i = n-2; i >= 0; i--){
      y_prime[i] = f[i] - g[i] * y_prime[i+1];
    }

    free_vector(f);
    free_vector(g);
  }
  else {
    char buff[1024];
    sprintf(buff,"spline_deriv: size=%d insufficient for calculation",n);
    push_error(buff, Error_General_);
  }
}


/*
void test_spline()
{
  int i;
  double x, y1, y2, y3;
  double xa[100], ya[100], y2a[100];
  int klocache = 0, khicache = 0;

  for(i=0; i<100; i++){
    xa[i] = i;
    ya[i] = 1./(1.+i);
  }

  spline(xa, ya, 100, y2a);

  for(x=0.; x<15.; x += 1.e-2){
    int klo, khi;
    interp_spline(xa, ya, y2a, 100, x, &y1, &klo, &khi);
    interp_spline_quick(xa, ya, y2a, klo, khi, x, &y2);
    
    printf("%g    %g  %g  %g   %g   %d  %d\n", 
	   x, y1, y2, y3, 1./(1.+x), klocache, khicache);
  }

  exit(0);
}
*/



void test_ratint()
{
  int i;

  double x, y, dy;
  double X[12], Y[12];

  X[0] = 0.;
  X[1] = 0.3;
  X[2] = 0.8;
  X[3] = 1.5;
  X[4] = 1.9;
  X[5] = 2.6;
  X[6] = 3.9;
  X[7] = 4.7;
  X[8] = 5.9;
  X[9] = 7.0;
  X[10] = 8.0;
  X[11] = 9.0;
  for(i=0; i<12; i++){
    Y[i] = 1./(1. + X[i]);
  }

  for(x=0.; x<8.; x += .1) {
    interp_rat(X, Y, 10, x, &y, &dy);
    printf("%g    %g  %g    %g\n", x, y, 1./(1.+x), dy);
  }

  exit(0);
}


/*
void test_polint()
{
  int i;

  double x, y, dy;
  double X[100], Y[100];

  for(i=0; i<100; i++){
    X[i] = i;
    Y[i] = 1./(1. + i);
  }

  for(x=0.; x<10.; x += .1){
    
    interp_poly(X, Y, 100, x, &y, &dy);

    printf("%g  %g  %g   %g\n", x, y, 1./(1.+x), dy);
  }

  exit(0);
}
*/
