/* Created: [GJ] Fri Apr 26 05:15:10 EDT 1996
 */
#ifndef GJIO_H
#define GJIO_H
#include <stdio.h>


/* Make up for missing i/o declarations in the Sparc/SunOS headers. */
#ifdef __sparc
extern int _filbuf(FILE *);
extern int fprintf(FILE *, const char *, ...);
extern int printf(const char *, ...);
extern int fclose(FILE *);
extern int sscanf(const char *, const char *, ...);
extern int ungetc(int, FILE *);
#endif


/* Open a file with minimal error checking and return a (FILE *).
 * Allows for an arbitrary file prefix/suffix given separately.
 * Returns (FILE *)0 on failure.
 */
extern FILE * openfile(const char * prefix,
		       const char * name,
		       const char * suffix,
		       const char * mode);


/* Get from a stream until reaching one of the chars in the
 * spacer string or EOF. Returns a (char *) string. Returns (char *)0
 * upon failure or EOF.
 * The stream is left with a terminator char in its mouth;
 * the terminator char is not eaten.
 * The returned string is allocated using new_string() and
 * should be freed by the client, using free_string().
 */
extern char * fgetuntil(FILE * fp, const char * terminators);


/* Get from a stream until finding a character which is NOT in
 * the given string. Therefore the result is a string which contains
 * only characters from the ignore string.
 * Compare to fgetuntil() above.
 * The returned string is allocated using new_string() and
 * should be freed by the client, using free_string().
 */
extern char * fgetuntilnot(FILE * fp, const char * ignore);


/* Gets the next line from a stream. Uses fgetuntil(),
 * but eats the "\n".
 * The returned string is allocated using new_string() and
 * should be freed by the client, using free_string().
 */
extern char * fgetnextline(FILE * fp);


/* Old style method to get a line from a stream and put
 * it in a buffer of size "size". Has the minor
 * advantage that it will work a little faster
 * than the above methods since there are no
 * alloc/realloc cycles, but has little else
 * to recommend it.
 */
extern int fgetline(char * buff, int size, FILE * fp);


/* Check if a linebuf is 'empty', ie commented out or
 * possessing only space characters.
 */
extern int emptyline(const char * linebuf, const char * commentchars);


/* Retrieve col'th numeric field from a linebuf. A non-numeric thing
 * counts as a column but does not generate an error unless it is in
 * the target column; special exception is made for the character '*',
 * which indicates a place holder for a column but no valid data ---
 * this should be dutifully ignored by data reading routines, and
 * getcol() returns 1 in that case.
 * If end of line before the column is found then return -1.
 * If everything was fine and we got a real data value then return 0.
 */
extern int getcolumn(double * number, const char * linebuf, int col);



#endif /* GJIO_H */

