/* 
   (C) 1997 Marco Falcioni

   template macros for lists

   methods:                 macro:

   new_P                    NEWLIST(LISTTYPE)
   delete_P                   FREELIST(LISTTYPE)
   print_P                  PRINTLIST(LISTTYPE, ELEMENT)
   clean_P                  CLEANLIST(LISTTYPE, ELEMENT)
   cat_P                    CATLIST(LISTTYPE, ELEMENT)
   test_P                   TESTLIST(LISTTYPE, ELEMENT)
   add_E                    ADDELEMENT(LISTTYPE, ELEMENT)
   remove_E                 REMOVEELEMENT(LISTTYPE, ELEMENT) 
   in_P                     INLIST(LISTTYPE, ELEMENT) 


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#define NEWLIST(LISTTYPE)                                   \
struct LISTTYPE * new_ ##LISTTYPE (void)                    \
{                                                           \
  struct LISTTYPE * me;                                     \
  me = (struct LISTTYPE *) malloc(sizeof(struct LISTTYPE)); \
                                                            \
  me->head = 0;                                             \
  me->tail = 0;                                             \
  me->length = 0;                                           \
                                                            \
  return me;                                                \
}                                                           \

#define FREELIST(LISTTYPE)                                                  \
void delete_ ##LISTTYPE (struct LISTTYPE * This){                             \
  if(This == 0) return;                                                     \
  if(This != 0 && This->length != 0) {                                      \
    push_generic_error                                                      \
      ("delete_" #LISTTYPE, "trying to free non empty " #LISTTYPE);           \
    exit_clean();                                                           \
  }                                                                         \
  free(This);                                                               \
  return;                                                                   \
}                                                                           \

/* we assume the element has a print_ELEMENT method */

#define PRINTLIST(LISTTYPE, ELEMENT)                                        \
void print_ ##LISTTYPE (const struct LISTTYPE * LISTTYPE, FILE * stream){   \
  long i;                                                                   \
  struct ELEMENT * a;                                                       \
                                                                            \
  if(LISTTYPE == 0) {                                                       \
       fprintf(stream, #LISTTYPE " is (nil)\n");                            \
    return;                                                                 \
  }	                                                                    \
  a = LISTTYPE->head;                                                       \
  if(a == 0) {                                                              \
    fprintf(stream, #LISTTYPE " is empty\n");                               \
    return;                                                                 \
  }									    \
                                                                            \
  fprintf(stream, #LISTTYPE " at %p {\n",LISTTYPE);                         \
  fprintf(stream, #LISTTYPE "->head = %p  ",LISTTYPE->head);                \
  fprintf(stream, #LISTTYPE "->tail = %p  ",LISTTYPE->tail);                \
  fprintf(stream, #LISTTYPE "->length = %ld\n",LISTTYPE->length);           \
                                                                            \
  for(i=0;i< LISTTYPE->length; i++){                                        \
    fprintf(stream,"%ld ",i);                                               \
    if(a == 0) {                                                            \
      push_generic_error                                                    \
    ("print_" #LISTTYPE, "end of " #LISTTYPE " reached unexpectedly");      \
 print_ ##ELEMENT (LISTTYPE->tail, stream);                                        \
      return;                                                         \
    }                                                                       \
    print_ ##ELEMENT ( a, stream);                            \
    a = a->next;                                                            \
  }                                                                         \
  fprintf(stream,"} end " #LISTTYPE ".\n");                                 \
  return;                                                                   \
} \

/* 
   wipe out all ELEMENTs in LISTTYPE: note that the macro needs
   to know the ELEMENT type to call the appropriate method
*/

#define CLEANLIST(LISTTYPE, ELEMENT)                     \
void clean_ ##LISTTYPE (struct LISTTYPE * a)             \
{                                                        \
  struct ELEMENT * b;                                    \
  if(a != 0 && a->length != 0) {                         \
    while(a->head != a->tail) {                          \
      b = a->tail;                                       \
      a->tail = b->prev;                                 \
      delete_ ##ELEMENT (b); }                             \
    delete_ ##ELEMENT (a->head);                           \
  }                                                      \
  a->head = 0;                                           \
  a->tail = 0;                                           \
  a->length = 0;                                         \
  return;                                                \
}                                                        \

/* 
   concatenate two strings: if both null;
   if one is null, return the other;
   if both empty return an empty LISTTYPE (and free the other one);
   empty and null--- see above;
   if one is not empty ant the other one is, free the empty one;
   by definition, a LISTTYPE of length zero is empty.
   otherwise attach head to tail and free one.

   the macro needs to know the ELEMENT type
*/

#define CATLIST(LISTTYPE, ELEMENT)                                          \
struct LISTTYPE * cat_ ##LISTTYPE (struct LISTTYPE * a, struct LISTTYPE * b)\
{                                                                           \
  if (a==b) return a;                                                       \
  if (a == 0 || b == 0) return (a == 0 ? b : a);                            \
  if (a->length == 0) {                                                     \
    delete_ ##LISTTYPE (a);                                                   \
    return b;                                                               \
  } else if(b->length == 0) {                                               \
    delete_ ##LISTTYPE (b);                                                   \
    return a;                                                               \
  }                                                                         \
                                                                            \
  a->tail->next = b->head;                                                  \
  b->head->prev = a->tail;                                                  \
                                                                            \
  a->tail = b->tail;                                                        \
  a->length += b->length;                                                   \
                                                                            \
  b->length = 0;                                                            \
  b->head = 0;                                                              \
  b->tail = 0;                                                              \
  delete_ ##LISTTYPE (b);                                                     \
  return a;                                                                 \
}                                                                           \


/* 
   what kind of liked list is This? 
*/

#define TESTLIST(LISTTYPE, ELEMENT)                                    \
void test_ ##LISTTYPE (const struct LISTTYPE * a, FILE * stream)       \
{                                                                      \
  unsigned long i;                                                     \
  struct ELEMENT * b;                                                  \
  if(a==0) {                                                           \
    fprintf(stream,"Null " #LISTTYPE " \n");                           \
    return;                                                            \
  }                                                                    \
  if(a->length == 0 && a->head == 0 && a->tail == 0) {                 \
    fprintf(stream,"Empty " #LISTTYPE " \n");                          \
    return;                                                            \
  }                                                                    \
  b = a->head;                                                         \
  for(i=0; i< a->length-1; i++) {                                      \
    b = b->next;                                                       \
    if(b==0) {                                                         \
      fprintf(stream,"Length not equal to number of elements\n");      \
      return;                                                          \
    }                                                                  \
  }                                                                    \
  fprintf(stream,"Healty " #LISTTYPE " \n");                           \
  return;                                                              \
}

/* appends ELEMENT after the last ELEMENT */

#define ADDELEMENT(LISTTYPE, ELEMENT)                                     \
void add_ ##ELEMENT (struct ELEMENT * me, struct LISTTYPE * to_me){       \
  if(me == 0 || to_me == 0) {                                             \
    push_generic_error("add_" #ELEMENT, "(nil) args passed");             \
    exit_clean();                                                         \
  }                                                                       \
  if(to_me->head == 0) {                                                  \
    to_me->head = me;                                                     \
    me->prev = 0;                                                         \
  } else {                                                                \
    to_me->tail->next = me;                                               \
    me->prev = to_me->tail;                                               \
  }                                                                       \
  me->next = 0;                                                           \
  to_me->tail = me;                                                       \
  to_me->length++;                                                        \
  return;                                                                 \
}

#define ADDELEMENTCHK(LISTTYPE, ELEMENT)                                  \
void add_ck_ ##ELEMENT (struct ELEMENT * me, struct LISTTYPE * to_me){\
  struct ELEMENT * L;                                                     \
  if(me == 0 || to_me == 0) {                                             \
    push_generic_error("add_" #ELEMENT "_check", "(nil) args passed");    \
    exit_clean();                                                         \
  }                                                                       \
                                                                          \
  if(to_me->length > 0) {                                                 \
    for (L = to_me->head; L != 0; L = L->next) {                          \
      if(compare_content_ ##ELEMENT(L,me)) {                              \
        delete_ ##ELEMENT(me);                                              \
        return;                                                           \
      }                                                                   \
    }                                                                     \
  }                                                                       \
                                                                          \
  if(to_me->head == 0) {                                                  \
    to_me->head = me;                                                     \
    me->prev = 0;                                                         \
  } else {                                                                \
    to_me->tail->next = me;                                               \
    me->prev = to_me->tail;                                               \
  }                                                                       \
                                                                          \
  me->next = 0;                                                           \
  to_me->tail = me;                                                       \
  to_me->length++;                                                        \
  return;                                                                 \
}                                                                         \

/* returns previous link if not (nil), next link otherwise, (nil) if 
   that is so */
#define REMOVEELEMENT(LISTTYPE, ELEMENT)                                 \
void remove_ ##ELEMENT (struct ELEMENT * me, struct LISTTYPE * from_me){ \
  if(me == 0 || from_me == 0) {                                          \
    push_generic_error("remove_" #ELEMENT, "(nil) args passed");         \
    exit_clean();                                                        \
  }                                                                      \
  if(in_ ##LISTTYPE (me, from_me)) {                                     \
    if(from_me->head == me) from_me->head = me->next;                    \
    if(from_me->tail == me) from_me->tail = me->prev;                    \
    me->prev->next = me->next;                                           \
    if(me->next !=0)  me->next->prev = me->prev;                         \
    delete_ ##ELEMENT (me);                                                \
    if(from_me->length != 0) from_me->length--;                          \
  } else {                                                               \
    push_generic_error                                                   \
  ("remove_" #ELEMENT,"trying to remove a nonexisting link");            \
  }   	   	   	   	   	   	   	   	         \
  return;                                                                \
}                                                                        \

/* checks a given ELEMENT belongs to the LISTTYPE 0 FALSE nonzero TRUE*/

#define INLIST(LISTTYPE, ELEMENT)                                    \
int in_ ##LISTTYPE (struct ELEMENT * me, struct LISTTYPE * in_me)    \
{                                                                    \
  struct ELEMENT * c;                                                \
    if(me == 0 || in_me == 0) {                                      \
    push_generic_error("in_" #LISTTYPE, "(nil) args passed");        \
    exit_clean();                                                    \
  }                                                                  \
  for(c = in_me->head; c != 0; c = c->next){                         \
    if(me == c) return 1;                                            \
  }                                                                  \
  return 0;                                                          \
}                                                                    \

/* quick sort, hoping we have short lists */
#define SORTLIST(LISTTYPE, ELEMENT)                                  \
void sort_ ##LISTTYPE (struct LISTTYPE * me)                          \
{                                                                    \
  struct ELEMENT * item;                                             \
  if(me == 0) {                                                      \
    push_generic_error("sort" #LISTTYPE,"(nil) arg passed");         \
    exit_clean();                                                    \
  }                                                                  \
  if(me->length == 0) return; /* empty chain */                      \
  for(item = me->head; item != 0; item = item->next) {               \
    struct ELEMENT * next;                                           \
    for(next = item->next; next != 0; next = next->next) {           \
      if(item->key > next->key) {                                    \
        copy_ ##ELEMENT(item, next);                                 \
      }                                                              \
    }                                                                \
  }                                                                  \
}                                                                    \
								     
