/*

  (C) 1997 Marco Falcioni

  how do we OOGL?


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "error.h"
#include "memory.h"
#include "matrices.h"
#include "interpolation.h"

#include "molecule.h"
#include "cell.h"
#include "grid.h"
#include "box.h"
#include "link.h"
#include "atoms.h"
#include "oogl.h"
#include "spline.h"
#include "params.h"
#include "positions.h"
#include "misc.h"
#include "functions.h"

/* local defs */

int oogl_status = 0;

FILE * stream = 0;

const struct Cell * C1;

double smooth(double a, double b);
/* local functions */
double oogl_framework(const struct Params * P,
		       struct Chain * Neigh,
		       struct Atom * A,
		      int nobonds,
		      FILE * stream);

void print_couple(double ** v, long a, long b);


void oogl_Cell(const struct Cell * C, int rec_vec)

{
  int i, c = 0;
  double ** corner = new_vector_dp(8);
  double * a = new_vector_d(3);

  if(!oogl_status) {
    push_generic_error("oogl_Cell", "call oogl_Init_list first");
    exit_clean();
  }

  for(i = 0; i < 8; i++) corner[i] = new_vector_d(3);

  for(i = 0; i < 2; i++) {
    long j;
    for(j = 0; j < 2; j++) {
      long k;
      for(k = 0; k < 2; k++) {
	a[0] = (double) k;
	a[1] = (double) j;
	a[2] = (double) i;
	
	matvec_multiply_d(corner[c], C->abc_to_xyz, a, 3);
	c++;
      }
    }
  }

  if(rec_vec) 
    fprintf(stream,"VECT\n15 30 2\n");
  else fprintf(stream,"VECT\n12 24 1\n");

  fprintf(stream,"2 2 2 2 2 2 2 2 2 2 2 2");

  if(rec_vec)  fprintf(stream," 2 2 2\n");
  else fprintf(stream,"\n");
  
  fprintf(stream,"1 0 0 0 0 0 0 0 0 0 0 0");

  if(rec_vec)  fprintf(stream," 1 0 0\n");
  else fprintf(stream,"\n");
  
  print_couple(corner, 0, 1);
  print_couple(corner, 0, 2);
  print_couple(corner, 0, 4);
  print_couple(corner, 1, 3);
  print_couple(corner, 1, 5);
  print_couple(corner, 2, 3);
  print_couple(corner, 2, 6);
  print_couple(corner, 3, 7);
  print_couple(corner, 4, 5);
  print_couple(corner, 4, 6);
  print_couple(corner, 5, 7);
  print_couple(corner, 6, 7);

  /* reciprocal lattice vectors (in red)*/
  if(rec_vec) {
    fprintf(stream,"0.1 0.1 0.1\n%6.3f %6.3f %6.3f\n",
	    100.*C->rec_vec[0][0]+.1, 100.*C->rec_vec[0][1]+.1, 
	    100.*C->rec_vec[0][2] +.1);
    fprintf(stream,"0.1 0.1 0.1\n%6.3f %6.3f %6.3f\n",
	    100.*C->rec_vec[1][0]+.1, 100.*C->rec_vec[1][1]+.1, 
	    100.*C->rec_vec[1][2]+.1);
    fprintf(stream,"0.1 0.1 0.1\n%6.3f %6.3f %6.3f\n",
	    100.*C->rec_vec[2][0]+.1, 100.*C->rec_vec[2][1]+.1, 
	    100.*C->rec_vec[2][2]+.1);
  }

  fprintf(stream, "0 0 0 1\n");

  if(rec_vec) {
    fprintf(stream, "1 0 0 1\n");
  }

  free_vector(a);
  for(i = 0; i < 8; i++) free_vector(corner[i]);
  free_vector(corner);

}

void print_couple(double ** v, long a, long b) {
  fprintf(stream,"%6.3f %6.3f %6.3f\n",v[a][0],v[a][1],v[a][2]);
  fprintf(stream,"%6.3f %6.3f %6.3f\n",v[b][0],v[b][1],v[b][2]);
}


void oogl_Boxes (const struct Box * B, const struct Cell * C, 
	      const struct Grid * G)
{
  int i;

  if(!oogl_status) {
    push_generic_error("oogl_Boxes", "call oogl_Init_list first");
    exit_clean();
  }

  if(B == 0 || C == 0 || G == 0) {
    push_generic_error("print_Box","(nil) args passed");
    exit_clean();
  }

  oogl_Box(B,C,G,0);
  for(i = 0; i < B->n; i++) {
    oogl_Box(B->near[i],C,G,(int) i+2);  
  }

  return;
}

void oogl_Positions(const struct Positions * T, 
		    const struct Cell * C, const int print_footer)
{
  int i;

  if(!oogl_status) {
    push_generic_error("oogl_Positions", "call oogl_Init_list first");
    exit_clean();
  }

  if(C == 0 || T == 0) {
    push_generic_error("oogl_Positons","(nil) args passed");
    exit_clean();
  }

  fprintf(stream, "INST transforms  { TLIST\n");

  for(i = 0; i < T->n_unique; i++) {
    struct Link * aa;
    for (aa = T->sym_related[i]->head; aa != 0; aa = aa->next) {
      double radius = aa->atom->Radius;
      mod_Atom(aa->atom);
      xyz_Atom(aa->atom, C);
      fprintf(stream, "%f 0 0 0 0 %f 0 0 0 0 %f 0 %f %f %f 1\n", 
	      radius, radius, radius, aa->atom->rx, aa->atom->ry, aa->atom->rz);
    }
  }
 
  if (print_footer) {
    fprintf(stream, "} geom {< dodec3.off}\n");
  }
  return;
}

void oogl_all_Molecules(const int Nmol, 
                        struct Molecule ** M,
                        const struct Cell * C, const int print_header)
{
  int i;

  if(!oogl_status) {
    push_generic_error("oogl_all_Molecules", "call oogl_Init_list first");
    exit_clean();
  }

  if (C == 0 || (Nmol && (M == 0))) {
    push_generic_error("oogl_Molecule","(nil) args passed");
    exit_clean();
  }

  if (print_header) fprintf(stream, "INST transforms  { TLIST\n");

  if (Nmol) {
    for (i = 0; i < Nmol; i++) {
      oogl_Molecule(M[i], C);
    }
  }

  fprintf(stream, "} geom {< dodec3.off}\n");

  return;
}


void oogl_Molecule(const struct Molecule * M, 
		    const struct Cell * C)
{
  int i;


  for(i = 0; i < M->N; i++) {

/* MWD Change */    
    fprintf(stream, "%f 0 0 0 0 %f 0 0 0 0 %f 0 %f %f %f 1\n", 
            0.5, 0.5, 0.5,
	    M->A[i]->rx, M->A[i]->ry, M->A[i]->rz);
  }
 
  return;
}



void oogl_Box(const struct Box * B, const struct Cell * C, 
	      const struct Grid * G, int a)
{
  struct Link * L;
  long color[3];
  double box[3];
  double c1 = 130,c2 = 200,c3 = 30;

  if(C == 0 || G == 0 || stream == 0) {
    push_generic_error("oogl_Box","(nil) args passed");
    exit_clean();
  }
  
  if(!oogl_status) {
    push_generic_error("oogl_Boxes", "call oogl_Init_list first");
    exit_clean();
  }


  if(B == 0) return;
  
  L = B->list->head;
  
  if(L == 0) return;

  index_atom_Grid(G, L->atom, C, color);
  
  box[0] = G->corner[0] + G->spacing * (double) color[0];
  box[1] = G->corner[1] + G->spacing * (double) color[1];
  box[2] = G->corner[2] + G->spacing * (double) color[2];
  if(a == 0) {
    color[0] = 0;
    color[1] = 0;
    color[2] = 0;
  } else {  
    color[0] = (long) fmod(a*c1+10*(Random() - 0.5), 256.0);
    color[1] = (long) fmod(a*c2+10*(Random() - 0.5), 256.0);
    color[2] = (long) fmod(a*c3+10*(Random() - 0.5), 256.0);
  }
  fprintf(stream,"box %f %f %f %f %ld %ld %ld\n",
	  box[0],box[1],box[2], G->spacing,
	  color[0],color[1],color[2]);
  
  
  for(L = B->list->head; L != 0; L = L->next) {
     fprintf(stream, "%f %f %f\n", 
	      L->atom->rx, L->atom->ry, L->atom->rz);
  }

  return;
}

void oogl_Grid(const struct Grid * grid, const struct Cell * C)
{
  int i;

  if(!oogl_status) {
    push_generic_error("oogl_Grid", "call oogl_Init_list first");
    exit_clean();
  }


  fprintf(stream, "INST transforms  { TLIST\n");

  for(i = 0; i < grid->n_boxes; i++) {
    struct Link * pp = grid->box_list[i]->list->head;

    if(pp == 0) continue; /* empty box */
    while (pp != 0) {
      double radius = pp->atom->Radius;
      xyz_Atom(pp->atom, C);
      fprintf(stream, "%f 0 0 0 0 %f 0 0 0 0 %f 0 %f %f %f 1\n", 
	      radius, radius, radius, pp->atom->rx, pp->atom->ry, pp->atom->rz);
      pp = pp->next;
    }
  }
  fprintf(stream, "}  geom {< ../dodec3.off}\n");
  return;
}

void oogl_Init_list(FILE * str) 
{
  if(str == 0) {
    push_generic_error("oogl_Init_list","(nil) stream passed");
    exit_clean();
  }
  if(oogl_status) {
    push_generic_error("oogl_Init_list","trying to init open oogl");
    exit_clean();  
  } else {
    oogl_status = 1;
    stream = str;
  }
  fprintf(stream, "(geometry atoms {LIST\n");
}

void oogl_Finish_list(void)
{
  if(oogl_status) {
    fprintf(stream, "})\n");
    fflush(stream);
    oogl_status = 0;
    stream = 0;
  } else {
    push_generic_error("oogl_Init_list","trying to finish no oogl");
    exit_clean();  
  }
  return;
}

void oogl_bonds(const struct Params * P, const struct Cell * C, 
		const struct Positions * T, const struct Grid * Gr, 
		int nobonds)
{
  struct Link * L;
  struct Chain * Neigh;

  C1 = C;

  Neigh = new_Chain();

  if(Gr == 0 || C == 0 || T == 0 || Gr->filled != 1) {
    push_generic_error("energy","(nil) or invalid args");
    exit_clean();
  }

  if(!oogl_status) {
    push_generic_error("oogl_bonds", "call oogl_Init_list first");
    exit_clean();
  }


  for(L = T->distinct->head; L != 0; L = L->next) {
    int i;
    struct Box * B;
    struct Atom * A = L->atom;
    struct Link * N;

    B = index_atom_Grid(Gr, A, C, 0); /* box where A lives */
    /* this info may eventually be cached in the Atom to save this call */

    if(A->n_bonds < 4) continue;

    /* first loop through atoms in the same box */
    for(N = B->list->head; N != 0; N = N->next) {
      double dist2;
      struct Link * newlink;
      if(A == N->atom || N->atom->n_bonds < 4) continue;
      dist2 = dist2_xyz(C, A, N->atom);

      if(dist2 < P->frange2 && dist2 > 0.0) {
	xyz_Atom(N->atom,C);
	newlink = new_Link(N->atom);
        newlink->key = sqrt(dist2);
	add_Link(newlink, Neigh);
      }
    }

    /* loop through atoms in the neighboring boxes */
    for(i = 0; i < B->n; i++) {
      for(N = B->near[i]->list->head; N != 0; N = N->next) {
	struct Link * newlink;
	double dist2;
	if(A == N->atom || N->atom->n_bonds < 4) continue; 
	dist2 = dist2_xyz(C, A, N->atom);
	if(dist2 < P->frange2 && dist2 > 0.0) {
	  xyz_Atom(N->atom,C);
	  newlink = new_Link(N->atom);
	  newlink->key = sqrt(dist2);
	  add_Link(newlink, Neigh);
	}
      }
    }

    oogl_framework(P, Neigh, A, nobonds, stream);
  
    clean_Chain(Neigh);

  } /* loop over distinct atoms */

  delete_Chain(Neigh);
  return;
}

void oogl_all_bonds(const struct Params * P, const struct Cell * C, 
		const struct Positions * T, const struct Grid * Gr, 
		int nobonds)
{
  int kk;
  struct Link * L;
  struct Chain * Neigh;

  C1 = C;

  Neigh = new_Chain();

  if(Gr == 0 || C == 0 || T == 0 || Gr->filled != 1) {
    push_generic_error("energy","(nil) or invalid args");
    exit_clean();
  }

  if(!oogl_status) {
    push_generic_error("oogl_bonds", "call oogl_Init_list first");
    exit_clean();
  }

  for(kk = 0; kk < Gr->n_boxes; kk++) {
    struct Box * B = Gr->box_list[kk];
    int i;

    struct Link * N;

    if(B == 0 || B->list->length == 0) continue;

    for(L = B->list->head; L != 0; L = L->next) {
    struct Atom * A = L->atom;

    if(A->n_bonds < 4) continue;

    xyz_Atom(A,C);
    
    /* first loop through atoms in the same box */
      for(N = B->list->head; N != 0; N = N->next) {
	double dist2;
	struct Link * newlink;

	if(L->atom == N->atom || N->atom->n_bonds < 4) continue;

	dist2 = dist2_xyz(C, A, N->atom);
	
	if(dist2 < P->frange2 ) {
	  xyz_Atom(N->atom, C);
	  newlink = new_Link(N->atom);
	  newlink->key = sqrt(dist2);
	  add_Link(newlink, Neigh);
	}
      }

      
    /* loop through atoms in the neighboring boxes */
      for(i = 0; i < B->n; i++) {
	for(N = B->near[i]->list->head; N != 0; N = N->next) {
	  struct Link * newlink;
	  double dist2;

	  if(L->atom == N->atom || N->atom->n_bonds < 4) continue;
	  dist2 = dist2_xyz(C, A, N->atom);
	  
	  if(dist2 < P->frange2 ) {
	    xyz_Atom(N->atom, C);
	    newlink = new_Link(N->atom);
	    newlink->key = sqrt(dist2);
	    add_Link(newlink, Neigh);
	  }
	}
      }

      oogl_framework(P, Neigh, A, nobonds, stream);
      
      clean_Chain(Neigh);
    }
  } /* loop over distinct atoms */

  delete_Chain(Neigh);
  return;
}

double oogl_framework(const struct Params * P,
		       struct Chain * Neigh,
		       struct Atom * A,
		      int nobonds,
		      FILE * stream)
{
  int i;
  int i_min[4];
  int blen = 0, nblen = 0;
  double min_ene = HUGE_VAL; /* in limits.h */
  double *BEne, *BNEne;
  double **Angles, **AEne;
  double * BD;
  double sixth = 1.0/6.0;
  struct Link * N;
  struct Atom ** B, **NB;
  char a1[1024];

  /* this string has the xyz position of the atom at the center of the framework */

  sprintf(a1, "%f %f %f ", A->rx, A->ry, A->rz);
    
  /* 
     these arrays have B: all the atoms close enough to be bonded
     NB the atoms close enough to feel the repulsive force 
     BD the distances of the bondable atoms, used to find the angles
  */

  B = (struct Atom **) malloc(Neigh->length * sizeof(struct Atom *));
  NB = (struct Atom **) malloc(Neigh->length * sizeof(struct Atom *));
  BD = new_vector_d(Neigh->length);
  
  for(N = Neigh->head; N != 0; N = N->next) {
    if(N->key < P->bond) {
      B[blen] = N->atom;
      BD[blen] = N->key;
      ++blen;
    } else {
      NB[nblen] = N->atom;
      ++nblen;
    }
  }


  if(blen < 5) {
    for(i = 0; i < blen; i++) {
      fprintf(stream, "VECT\n1 2 1\n2\n1\n%s\n%f %f %f\n0 1 0 1\n", /* GREEN */
	      a1, (B[i]->rx + A->rx) * .5, (B[i]->ry + A->ry) * .5, (B[i]->rz + A->rz)*.5);
    }
    goto second; /* we skip the minimum energy search */
  }

 /* non bonded Atoms */

  if(nobonds) /* if this is true we do draw them */
  for(i = 0; i < nblen; i++) {
    fprintf(stream, "VECT\n1 2 1\n2\n1\n%s\n%f %f %f\n0 0 1 1\n",  /* BLUE */
	    a1, NB[i]->rx, NB[i]->ry, NB[i]->rz );

  }
 
  /* we need minimum search */

  BEne = new_vector_d(blen); /* energies associated to actual bonds */
  BNEne = new_vector_d(blen); /* energies associated to non-bonds
				 for atoms close enough to be bonded */
  Angles = new_matrix_d(blen); /* holds all possible angles */
  AEne = new_matrix_d(blen); /* holds the energies associaded with them */
 
  /* we first fill the arrays with the angles and energies: 
     we include all possible pairs */
  for(i = 0; i < blen; i++) {
    int j;
    int k,m;
    double over = 0.5, factor;

    if(B[i]->root == A->root) {
      over = 1.0;
    }

    BEne[i] = over * interp_spline(P->Bond->X, P->Bond->Y, P->Bond->Y2, 
			    P->Bond->n, BD[i], &k, &m);
    BNEne[i] = over * interp_spline(P->NoBond->X, P->NoBond->Y, P->NoBond->Y2, 
			    P->NoBond->n, BD[i], &k, &m);

    factor = smooth(BD[i],P->bond);

    /* we only need to fill half the matrices, since they are symmetric. */
    for(j = 0; j < blen; j++) {
      double angle = find_angle(A, B[i], B[j], BD[i], BD[j]); 
      double factor2;
      if(i >= j) continue;
      
      factor2 = smooth(BD[j],P->bond);
      
      Angles[i][j] = Angles[j][i] = angle;
      AEne[i][j] = AEne[j][i] = factor * factor2 * 
	interp_spline(P->Angle->X, P->Angle->Y, P->Angle->Y2, 
		      P->Angle->n, angle, &k, &m);
    }
    Angles[i][i] = 0.0;
    AEne[i][i] = 0.0;
  }
  
  for(i = 0; i < blen; i++) {
    int j;
    for(j  = 0; j < blen; j++) {
      int k;
      if(j >= i) continue;
      for(k = 0; k < blen; k++) {
	int m;
	if(k >= j && k >= i) continue; 
	for(m = 0; m < blen; m++) {
	  double energy = 0.0;
	  double av_angle = 0.0;
	  int n;
	  int a,b;
	  /* again, we only scan half the cases in each dimension, since 
	     everything is symmetric under relabeling */
	  if(m >= k || m >= i || m >= j) continue;
	  /* add contribution of close enough 
	     nonbonded atoms (overcounted) */
	  for(n = 0; n < blen; n++) {
	    if(n != i && n != j && n != k && n != m) {
	      energy += P->wt_nobond * BNEne[n];
	    }
	  } 
	  /* contribution of bonded atoms (overcounted) */
	  energy += P->wt_bond * 
	    (BEne[i] + BEne[j] + BEne[k] + BEne[m]);
	  /* contribution of angle energy */
	  energy += P->wt_angl * (AEne[i][j] + AEne[i][k] + AEne[i][m] + 
	    AEne[j][k] + AEne[j][m] + AEne[k][m]);
	  av_angle = Angles[i][j] + Angles[i][k] + Angles[i][m] + 
	    Angles[j][k] + Angles[j][m] + Angles[k][m];
	  av_angle *= sixth;
	  energy += P->wt_aang * interp_spline(P->AvAngle->X, P->AvAngle->Y, 
				  P->AvAngle->Y2, P->AvAngle->n, 
				  av_angle, &a, &b);

	  if(energy < min_ene) {
	    min_ene = energy;
	    i_min[0] = i;
	    i_min[1] = j;
	    i_min[2] = k;
	    i_min[3] = m;
	  }
	}
      }
    }
  }

  for(i = 0; i < blen; i++) {
    if(i == i_min[0] || i == i_min[1] ||i == i_min[2] ||i == i_min[3]) {
      fprintf(stream, "VECT\n1 2 1\n2\n1\n%s\n%f %f %f\n0 1 0 1\n", /* GREEN */
	      a1, (B[i]->rx + A->rx)*.5, (B[i]->ry + A->ry)*.5, (B[i]->rz + A->rz)*.5);
    } else {
      fprintf(stream, "VECT\n1 2 1\n2\n1\n%s\n%f %f %f\n0 0 1 1\n", /* BLUE */
	      a1, (B[i]->rx + A->rx)*.5, (B[i]->ry + A->ry)*.5, (B[i]->rz + A->rz)*.5);

    }
  }

  free_vector(BEne);
  free_vector(BNEne);
  free_matrix_d(Angles, blen);
  free_matrix_d(AEne, blen);
 second:
  free_vector(BD);
  free(B);
  free(NB);

  return min_ene;

}


