/*

  Marco Falcioni (C) 1997

  data and routines for handling t atoms

  the Positions Structure holds the position of the atoms, the number of
  unique atoms, the number of symmetry ops and the number of total atoms, 
  in case some of them are on special positions.  It must be safe to assume
  that all functions return consistent positions, i.e. we can only move the 
  unique atoms, but all the other ones must follow.


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __positions_h
#define __positions_h

struct Chain;
struct Atom;
struct Molecule;

/* data */

struct Positions {
  struct Chain * unique;
  struct Chain * distinct;
  struct Chain ** sym_related;
  unsigned long n_unique;
  unsigned long n_symmetry;
  unsigned long n_actual; /* some atoms may sit in special positions */
  unsigned density; /* this is the number of T atoms */
  unsigned test; /* sometimes we leave positions not symmetrized */
  unsigned mod;
  unsigned grid;
  unsigned merged;
  struct Molecule ** M;
  int Nmol, Nmol_atoms;
};

/* function prototypes */

struct Positions * new_Positions(unsigned long n_unique, 
				 unsigned long n_symmetry,
				 unsigned long n_actual,
                                 unsigned long Nmol,
                                 struct Molecule ** M);


void delete_Positions(struct Positions * T);

void copy_Positions(const struct Positions * T_o, struct Positions * T_n);
void clone_Positions(const struct Positions * T_o, struct Positions * T_n);

/* bring all Positions inside the unit cell */
void mod_Positions(struct Positions * T);

/* randomize the positions */
void rand_Positions(struct Positions * T);

/* some basic I/O */
void read_Positions(struct Positions * T, FILE * stream);
void read_Positions_XTL(struct Positions * T, FILE * stream);
void print_unique_Positions (const struct Positions * T, FILE * stream);
void write_unique_Positions (const struct Positions * T, FILE * stream);
void write_unique_Positions_XTL (const struct Positions * T, FILE * stream);
void write_Positions (const struct Positions * T, FILE * stream);
void print_Positions (const struct Positions * T, FILE * stream);
struct Positions * init_Positions (FILE * stream);
char * sprint_Positions(const struct Positions * T, char * Ch);
#endif
