/*

  Marco Falcioni (C) 1997

  Powder X-Ray or Neutron Diffraction Data


  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html



*/

#ifndef __rad_pattern_h
#define __rad_pattern_h

/* includes */
#include <stdio.h>

#include "misc.h"

enum Rad_type;
struct Spot;
struct Cell;
struct Rad_pattern {
  struct Spot ** I;
  double wavelength;
  double wavelength2;
  double xarea;
  double narea;
  unsigned length;
  double ** xray_corr;
  double ** neu_corr;
  enum Rad_type type;
  int corr;
  int composites;
  int angles;
};

enum  Normalize {CALC, VALUE, ALL};

/* function prototypes */
struct Rad_pattern * new_Rad_pattern(void);
void delete_Rad_pattern(struct Rad_pattern * me);
void read_Rad_pattern(struct Rad_pattern * me, FILE * stream);
void print_Rad_pattern(const struct Rad_pattern * me, FILE * stream);
void write_Rad_pattern(const struct Rad_pattern * me, FILE * stream);
void scale_Rad_pattern(struct Rad_pattern * me);
void sort_Rad_pattern(struct Rad_pattern * me);
double distance_Rad_pattern(struct Rad_pattern * me);
void sum_composites_Rad_pattern(struct Rad_pattern * RP);
void plot_Rad_pattern(const struct Rad_pattern * me, 
		      const struct Cell * C, FILE * stream);
void norm_Rad_pattern(struct Rad_pattern * RP, enum Normalize);
void angle_Rad_pattern(struct Rad_pattern * RP, const struct Cell * C); 
void copy_Rad_pattern(struct Rad_pattern * RP);
#endif
