/*

  Marco Falcioni (C) 1997

  read a datafile in a standard form

  (C) 1997 Marco Falcioni (falcioni@ucla.edu)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
  USA.

  http://www.fsf.org/copyleft/gpl.html

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "interpolation.h"
#include "positions.h"
#include "cell.h"
#include "symmetry.h"
#include "misc.h"
#include "functions.h"
#include "params.h"
#include "rad_pattern.h"
#include "SP_entry.h"
#include "spline.h" 

void read_data(struct Params ** P, struct Positions ** T, 
	       struct Cell ** C, struct Sym_group ** G, 
	       struct Rad_pattern ** RP,
	       FILE * stream)
{
  int xa, xb;
  double amount;

  if(stream == 0) {
    push_generic_error("read_data","(nil) arg passed");
    exit_clean();
  }

  (*P) = read_Params(stream);

  (*T) = init_Positions(stream);
  
  (*C) = read_Cell(stream);

  (*G) = read_Sym_group((*T)->n_symmetry, stream);

  if((*C)->csym == is_csym_Sym_group (*G)) {
    /* no problem */
  } else {
    push_generic_error("read_data", "Centrosymmetric group/cell mismatch: check input");
    exit_clean();
  }


  read_Positions(*T, stream);
  
  (*P)->Bond = read_Spline((*P)->Bond_Spl_file);
  (*P)->NoBond = read_Spline((*P)->NoBond_Spl_file);
  (*P)->Angle = read_Spline((*P)->Angle_Spl_file);
  (*P)->AvAngle = read_Spline((*P)->AvAngle_Spl_file);
  (*P)->SP = read_SP_file(*T, (*P)->Scat_Fact_file);

  if((*P)->Rad_file != 0) {
    FILE * a;
    *RP = new_Rad_pattern();
    a = fopen((*P)->Rad_file, "r");
    if(a == 0) { /* try to read from the data path */
      char file[200];
      strcpy(file, (*P)->data_path); 
      strcat(file, (*P)->Rad_file);
      a = safe_open(file, "r"); /* if this does not work we bail out */
    }
    read_Rad_pattern(*RP, a);
    fclose(a);
    get_correction(*RP, *T, *C, (*P)->SP);
  }

  /*  test_Sym_group(*G); */

  (*G)->test = 1;

  if((*C)->csym == 0) {
    find_polar_Sym_group(*G);
  }

  /* 
     smooth the "force fields" at the distance cut-off.
     we subtract the value at the maximum distance from the 
     curve.
  */

  /* evaluate the Bond spline at the cut-off and shift it */

  amount = interp_spline((*P)->Bond->X, (*P)->Bond->Y, (*P)->Bond->Y2, 
			 (*P)->Bond->n, (*P)->bond, &xa, &xb);

  shift_Spline((*P)->Bond, - amount);
  
  /* evaluate the NoBond spline at the cut-off and shift it */
  
  amount = interp_spline((*P)->NoBond->X, (*P)->NoBond->Y, (*P)->NoBond->Y2, 
			 (*P)->NoBond->n, (*P)->frange, &xa, &xb);

  shift_Spline((*P)->NoBond, - amount);

  return;

}
